/*
 *  cjoboard.c
 *  Othello
 *
 *  Created by Curtis Jones on 2010.09.27.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "cjoboard.h"
#include "logger.h"
#include <stdint.h>
#include <string.h>





#pragma mark -
#pragma mark Prototypes

//
// looks in a direction for pieces to flip; does not flip them.
//
static inline uint32_t __cjoboard_playcnt_n  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_w  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_s  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_e  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_nw (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_ne (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_sw (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_playcnt_se (cjoboard_t*, cjospace_t, cjostate_t);

//
// looks in a direction for pieces to flip; flips them.
//
static inline uint32_t __cjoboard_play_n  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_w  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_s  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_e  (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_nw (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_ne (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_sw (cjoboard_t*, cjospace_t, cjostate_t);
static inline uint32_t __cjoboard_play_se (cjoboard_t*, cjospace_t, cjostate_t);





#pragma mark -
#pragma mark Structors

/**
 * Zero out the board. If there were additional memory to allocate for the board, this is where 
 * we'd do it.
 */
inline int
cjoboard_init (cjoboard_t *board)
{
	if (unlikely(board == NULL))
		LOG_ERROR_AND_RETURN(-1, "null cjoboard_t");
	
	memset(board, 0, sizeof(cjoboard_t));
	
	for (int i = 0; i < 64; ++i)
		board->board[i] = CJO_STATE_NULL;
	
	CJO_BOARD_SET_STATE(board, CJO_SPACE_D4, CJO_STATE_BLACK);
	CJO_BOARD_SET_STATE(board, CJO_SPACE_D5, CJO_STATE_WHITE);
	CJO_BOARD_SET_STATE(board, CJO_SPACE_E4, CJO_STATE_WHITE);
	CJO_BOARD_SET_STATE(board, CJO_SPACE_E5, CJO_STATE_BLACK);
	
	return 0;
}





#pragma mark -
#pragma mark Accessors

/**
 *
 *
 */
inline int
cjoboard_play (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate;
	uint32_t wins = 0;
	
	if (unlikely(board == NULL))
		LOG_ERROR_AND_RETURN(-1, "null cjoboard_t");
	
	// verify that the move is black or white; not null
	if (unlikely(state == CJO_STATE_NULL))
		LOG_ERROR_AND_RETURN(-2, "can not play null move");
	
	// verify that the target space has not yet been play
	if (unlikely(CJO_BOARD_GET_STATE(board,space) != CJO_STATE_NULL))
		LOG_ERROR_AND_RETURN(-3, "can not play at space 0x%05x", space);
	
	// if we're playing white, get black; if we're playing black, get white
	xstate = CJO_XSTATE(state);
	
	// north
	if (CJO_SPACE_ROW_GT(space,CJO_SPACE_ROW_B))
		wins += __cjoboard_play_n(board, space, state);
	
	// south
	if (CJO_SPACE_ROW_LT(space,CJO_SPACE_ROW_G))
		wins += __cjoboard_play_s(board, space, state);
	
	// west
	if (CJO_SPACE_COL_GT(space,CJO_SPACE_COL_2))
		wins += __cjoboard_play_w(board, space, state);
	
	// east
	if (CJO_SPACE_COL_LT(space,CJO_SPACE_COL_7))
		wins += __cjoboard_play_e(board, space, state);
	
	// northwest
	if (CJO_SPACE_ROW_GT(space,CJO_SPACE_ROW_B) && CJO_SPACE_COL_GT(space,CJO_SPACE_COL_2))
		wins += __cjoboard_play_nw(board, space, state);
	
	// northeast
	if (CJO_SPACE_ROW_GT(space,CJO_SPACE_ROW_B) && CJO_SPACE_COL_LT(space,CJO_SPACE_COL_7))
		wins += __cjoboard_play_ne(board, space, state);
	
	// southwest
	if (CJO_SPACE_ROW_LT(space,CJO_SPACE_ROW_H) && CJO_SPACE_COL_GT(space,CJO_SPACE_COL_2))
		wins += __cjoboard_play_sw(board, space, state);
	
	// southeast
	if (CJO_SPACE_ROW_LT(space,CJO_SPACE_ROW_H) && CJO_SPACE_COL_LT(space,CJO_SPACE_COL_7))
		wins += __cjoboard_play_se(board, space, state);
	
	// if the player can't take any pieces by playing at this space, then the move is invalid
	if (wins != 0)
		CJO_BOARD_SET_STATE(board,space,state)
	else
		LOG_ERROR_AND_RETURN(-4, "not a valid play at space 0x%05x for player %d", space, state);
	
	return wins;
}

/**
 * Returns the number of plays available on the board for the given user. For each position on the
 * board that is empty, if the space is playable (ie, we can flip pieces), then increment the
 * "plays" count and continue on to the next position on the board.
 */
inline int
cjoboard_playcnt (cjoboard_t *board, cjostate_t state)
{
	uint32_t wins=0, playcnt=0;
	cjospace_t space;
	
	for (int i = 0; i < 64; ++i) {
		space = cjo_pos_2_space[i].val;
		wins = 0;
		
		if (CJO_BOARD_GET_STATE(board,space) != CJO_STATE_NULL)
			continue;
		
		wins += __cjoboard_playcnt_n(board, space, state);
		wins += __cjoboard_playcnt_s(board, space, state);
		wins += __cjoboard_playcnt_w(board, space, state);
		wins += __cjoboard_playcnt_e(board, space, state);
		wins += __cjoboard_playcnt_nw(board, space, state);
		wins += __cjoboard_playcnt_ne(board, space, state);
		wins += __cjoboard_playcnt_sw(board, space, state);
		wins += __cjoboard_playcnt_se(board, space, state);
		
		if (wins)
			playcnt += 1;
	}
	
	return playcnt;
}

/**
 *
 *
 */
inline int
cjoboard_playlst (cjoboard_t *board, cjostate_t state, cjoplays_t *plays)
{
	uint32_t totwins=0, poswins, tmpwins, dirs;
	cjospace_t space;
	
	if (unlikely(plays == NULL))
		LOG_ERROR_AND_RETURN(-1, "null cjoplays_t");
	
	for (int i = 0; i < 64; ++i) {
		space = cjo_pos_2_space[i].val;
		poswins = 0;
		dirs = 0;
		
		if (CJO_BOARD_GET_STATE(board,space) != CJO_STATE_NULL)
			continue;
		
		if (0 < (tmpwins = __cjoboard_playcnt_n(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_N;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_s(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_S;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_w(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_E;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_e(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_W;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_nw(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_NW;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_ne(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_NE;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_sw(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_SW;
		}
		
		if (0 < (tmpwins = __cjoboard_playcnt_se(board,space,state))) {
			totwins += tmpwins;
			poswins += tmpwins;
			dirs |= CJO_DIR_SE;
		}
		
		if (dirs != 0)
			plays->plays[plays->playcnt++] = (cjoplay_t){ space, dirs, poswins };
	}
	
	plays->wins = totwins;
	
	return plays->playcnt;
}

/**
 *
 *     1 2 3 4 5 6 7 8
 *    -----------------
 *  A | | | | | | | | | A
 *  B | | | | | | | | | B
 *  C | | | | | | | | | C
 *  D | | | |x|o| | | | D
 *  E | | | |o|x| | | | E
 *  F | | | | | | | | | F
 *  G | | | | | | | | | G
 *  H | | | | | | | | | G
 *    -----------------
 *     1 2 3 4 5 6 7 8
 *
 *     black: 2 / 4
 *     white: 2 / 4
 *
 */
void
cjoboard_print (cjoboard_t *board)
{
	printf("\n");
	printf("    1 2 3 4 5 6 7 8\n");
	printf("   -----------------\n");
	
	printf(" A |%c|%c|%c|%c|%c|%c|%c|%c| A\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_A8)));
	
	printf(" B |%c|%c|%c|%c|%c|%c|%c|%c| B\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_B8)));
	
	printf(" C |%c|%c|%c|%c|%c|%c|%c|%c| C\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_C8)));
	
	printf(" D |%c|%c|%c|%c|%c|%c|%c|%c| D\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_D8)));
	
	printf(" E |%c|%c|%c|%c|%c|%c|%c|%c| E\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_E8)));
	
	printf(" F |%c|%c|%c|%c|%c|%c|%c|%c| F\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_F8)));
	
	printf(" G |%c|%c|%c|%c|%c|%c|%c|%c| G\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_G8)));
	
	printf(" H |%c|%c|%c|%c|%c|%c|%c|%c| H\n",
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H1)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H2)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H3)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H4)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H5)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H6)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H7)),
				 CJO_STATE_CHAR(CJO_BOARD_GET_STATE(board,CJO_SPACE_H8)));
	
	printf("   -----------------\n");
	printf("    1 2 3 4 5 6 7 8\n");
	printf("\n");
	printf("    black: %d / %d\n", (int)board->black, cjoboard_playcnt(board,CJO_STATE_BLACK));
	printf("    white: %d / %d\n", (int)board->white, cjoboard_playcnt(board,CJO_STATE_WHITE));
	printf("\n");
}





#pragma mark -
#pragma mark Private

/**
 * Looks NORTH for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_n (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_ROW_GT(space,CJO_SPACE_ROW_B))
		return 0;
	
	// repeat as long as we're seeing the opposite player's pieces; and count them. stop as soon as
	// we see one of our own pieces or an empty space. also, stop after we hit the top row.
	do {
		CJO_SPACE_MOVE_N(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_ROW(space2) != CJO_SPACE_ROW_A);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks SOUTH for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_s (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_ROW_LT(space,CJO_SPACE_ROW_G))
		return 0;
	
	do {
		CJO_SPACE_MOVE_S(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_ROW(space2) != CJO_SPACE_ROW_H);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks WEST for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_w (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_COL_GT(space,CJO_SPACE_COL_2))
		return 0;
	
	do {
		CJO_SPACE_MOVE_W(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_COL(space2) != CJO_SPACE_COL_1);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks EAST for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_e (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_COL_LT(space,CJO_SPACE_COL_7))
		return 0;
	
	do {
		CJO_SPACE_MOVE_E(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_COL(space2) != CJO_SPACE_COL_8);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks NORTHWEST for pieces to flip, but does not them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_nw (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_ROW_GT(space,CJO_SPACE_ROW_B) || !CJO_SPACE_COL_GT(space,CJO_SPACE_COL_2))
		return 0;
	
	do {
		CJO_SPACE_MOVE_NW(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_ROW(space2) != CJO_SPACE_ROW_A && CJO_SPACE_COL(space2) != CJO_SPACE_COL_1);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks NORTHEAST for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_ne (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_ROW_GT(space,CJO_SPACE_ROW_B) || !CJO_SPACE_COL_LT(space,CJO_SPACE_COL_7))
		return 0;
	
	do {
		CJO_SPACE_MOVE_NE(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_ROW(space2) != CJO_SPACE_ROW_A && CJO_SPACE_COL(space2) != CJO_SPACE_COL_8);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks SOUTHWEST for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_sw (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_ROW_LT(space,CJO_SPACE_ROW_H) || !CJO_SPACE_COL_GT(space,CJO_SPACE_COL_2))
		return 0;
	
	do {
		CJO_SPACE_MOVE_SW(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_ROW(space2) != CJO_SPACE_ROW_H && CJO_SPACE_COL(space2) != CJO_SPACE_COL_1);
	
	return match == 1 ? xcount : 0;
}

/**
 * Looks SOUTHEAST for pieces to flip, but does not flip them.
 *
 */
static inline uint32_t
__cjoboard_playcnt_se (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjospace_t space2 = space;
	cjostate_t state2, xstate = CJO_XSTATE(state);
	int match=0, xcount=0;
	
	if (!CJO_SPACE_ROW_LT(space,CJO_SPACE_ROW_H) || !CJO_SPACE_COL_LT(space,CJO_SPACE_COL_7))
		return 0;
	
	do {
		CJO_SPACE_MOVE_SE(space2);
		
		state2 = CJO_BOARD_GET_STATE(board,space2);
		
		if (state2 == xstate)
			xcount++;
		else if (state2 == state) {
			match = 1;
			break;
		}
		else
			break;
	}
	while (CJO_SPACE_ROW(space2) != CJO_SPACE_ROW_H && CJO_SPACE_COL(space2) != CJO_SPACE_COL_8);
	
	return match == 1 ? xcount : 0;
}









/**
 * Looks NORTH for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_n (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount = 0;
	
	xcount = __cjoboard_playcnt_n(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_N(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks SOUTH for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_s (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount = 0;
	
	xcount = __cjoboard_playcnt_s(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_S(space);
			
			if (CJO_BOARD_GET_STATE(board, space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks WEST for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_w (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount;
	
	xcount = __cjoboard_playcnt_w(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_W(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks EAST for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_e (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount;
	
	xcount = __cjoboard_playcnt_e(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_E(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks NORTHWEST for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_nw (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount;
	
	xcount = __cjoboard_playcnt_nw(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_NW(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board, space, state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks NORTHEAST for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_ne (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount;
	
	xcount = __cjoboard_playcnt_ne(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_NE(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks SOUTHWEST for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_sw (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount;
	
	xcount = __cjoboard_playcnt_sw(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_SW(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}

/**
 * Looks SOUTHEAST for pieces to flip, and flips them.
 *
 */
static inline uint32_t
__cjoboard_play_se (cjoboard_t *board, cjospace_t space, cjostate_t state)
{
	cjostate_t xstate = CJO_XSTATE(state);
	int xcount;
	
	xcount = __cjoboard_playcnt_se(board, space, state);
	
	if (xcount != 0) {
		while (1) {
			CJO_SPACE_MOVE_SE(space);
			
			if (CJO_BOARD_GET_STATE(board,space) == xstate)
				CJO_BOARD_SET_STATE(board,space,state)
			else
				break;
		}
	}
	
	return xcount;
}
